import qs from 'qs';
export var debounce = function (delay, fn) {
    var timerId;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (timerId) {
            clearTimeout(timerId);
        }
        timerId = setTimeout(function () {
            fn.apply(void 0, args);
            timerId = null;
        }, delay);
    };
};
export var parseQuery = function (queryString) {
    return qs.parse(queryString, {
        ignoreQueryPrefix: true,
        arrayFormat: 'indices',
        encodeValuesOnly: true,
        arrayLimit: 150,
        depth: 6,
        decoder: function (value) {
            if (/^-?(0|0\.\d+|[1-9]\d+(\.\d+)?)$/.test(value)) {
                return parseFloat(value);
            }
            var keywords = {
                undefined: undefined,
                true: true,
                false: false,
                null: null
            };
            if (value in keywords) {
                return keywords[value];
            }
            return decodeURIComponent(value);
        }
    });
};
// isObject : a -> Boolean
var isObject = function (x) {
    return !!x && Object.prototype.toString.call(x) === '[object Object]';
};
var isArray = function (x) { return !!x && Array.isArray(x); };
var ifNotEqualsThenAssign = function (obj) { return function (obj2) { return function (key) {
    if (obj[key] !== obj2[key]) {
        obj[key] = obj2[key];
    }
}; }; };
export var deepMerge = function (obj) { return function (obj2) {
    if (isArray(obj) && isArray(obj2)) {
        obj.splice(obj2.length, obj.length - obj2.length);
    }
    if (isObject(obj) && isObject(obj2)) {
        for (var key in obj) {
            if (!obj2[key])
                delete obj[key];
        }
    }
    for (var key in obj2) {
        if (typeof obj[key] !== 'object' || typeof obj2[key] !== 'object') {
            ifNotEqualsThenAssign(obj)(obj2)(key);
            continue;
        }
        if (isObject(obj2[key]) || isArray(obj2[key])) {
            deepMerge(obj[key])(obj2[key]);
        }
    }
}; };
//# sourceMappingURL=utils.js.map