import { observe } from '@nx-js/observer-util';
import qs from 'qs';
import { debounce, parseQuery as _parseQuery, deepMerge } from './utils';
import { batch } from 'react-easy-state';

let shouldReact = true;

const serializeModelReactions = debounce(0, reaction => {
	if (shouldReact) {
		return reaction();
	}
	shouldReact = true;
});

type RouterSetup = {
	model;
	serializeModelToUrl;
	deserializeUrlToModel;
	parseQuery?;
	onPopState?;
};

export const routerSetup = ({
	model,
	serializeModelToUrl,
	deserializeUrlToModel,
	parseQuery,
	onPopState
}: RouterSetup) => {
	observe(
		() => {
			const { path, query } = serializeModelToUrl(model);
			window.history.pushState(
				null,
				path,
				`/${path}?${qs.stringify(query, {
					arrayFormat: 'indices',
					encodeValuesOnly: true
				})}`
			);
		},
		{ scheduler: serializeModelReactions }
	);
	window.onpopstate = () => {
		shouldReact = false;
		const urlModel = deserializeUrlToModel({
			path: location.pathname.slice(1),
			query:
				(parseQuery && parseQuery(location.search)) ||
				_parseQuery(location.search)
		});
		batch(() => {
			for (const key in urlModel) {
				deepMerge(model[key])(urlModel[key]);
			}
			if (onPopState) {
				onPopState(model);
			}
		});
	};
};
