import { store } from 'react-easy-state';
import { prepareIntents } from './intents';
import { napSetup } from './nap';
import { routerSetup } from './router';

type Proposal<A> = {
	type: A;
	payload?;
};

type Present<A> = (proposal: Proposal<A>) => void;
type Reducer<A extends string> = { [key in A | 'default']: any };

type Props<M extends object, A extends string> = {
	model: {
		model: M;
		onModelInit: (model: M) => void;
	};
	reducer: {
		reducer: Reducer<A>;
		onReduce: (proposal: Proposal<A>, model: M) => void;
	};
};

export const createStore = <M extends object, A extends string>({
	model: { model, onModelInit },
	reducer: { reducer, onReduce }
}: Props<M, A>) => {
	const _model = store(model);
	if (onModelInit) {
		onModelInit(_model);
	}

	const present: Present<A> = proposal => {
		const acceptorType = reducer[proposal.type];
		if (onReduce) {
			onReduce(proposal, _model);
		}
		return acceptorType
			? acceptorType(proposal.payload)
			: reducer.default(proposal);
	};

	const createIntents = prepareIntents(present);

	return { createIntents, napSetup, routerSetup, model: _model };
};
