import qs from 'qs';

export const debounce = (delay, fn) => {
	let timerId;
	return (...args) => {
		if (timerId) {
			clearTimeout(timerId);
		}
		timerId = setTimeout(() => {
			fn(...args);
			timerId = null;
		}, delay);
	};
};

export const parseQuery = queryString => {
	return qs.parse(queryString, {
		ignoreQueryPrefix: true,
		arrayFormat: 'indices',
		encodeValuesOnly: true,
		arrayLimit: 150,
		depth: 6,
		decoder(value) {
			if (/^-?(0|0\.\d+|[1-9]\d+(\.\d+)?)$/.test(value)) {
				return parseFloat(value);
			}

			const keywords = {
				undefined,
				true: true,
				false: false,
				null: null
			};
			if (value in keywords) {
				return keywords[value];
			}

			return decodeURIComponent(value);
		}
	});
};

// isObject : a -> Boolean
const isObject = x =>
	!!x && Object.prototype.toString.call(x) === '[object Object]';

const isArray = x => !!x && Array.isArray(x);

const ifNotEqualsThenAssign = obj => obj2 => key => {
	if (obj[key] !== obj2[key]) {
		obj[key] = obj2[key];
	}
};

export const deepMerge = obj => obj2 => {
	if (isArray(obj) && isArray(obj2)) {
		obj.splice(obj2.length, obj.length - obj2.length);
	}
	if (isObject(obj) && isObject(obj2)) {
		for (const key in obj) {
			if (!obj2[key]) delete obj[key];
		}
	}

	for (const key in obj2) {
		if (typeof obj[key] !== 'object' || typeof obj2[key] !== 'object') {
			ifNotEqualsThenAssign(obj)(obj2)(key);
			continue;
		}
		if (isObject(obj2[key]) || isArray(obj2[key])) {
			deepMerge(obj[key])(obj2[key]);
		}
	}
};
